function [out]=NL_S_AESDecryption(s,in)
	[lhs,rhs]=argn();
	lhsn=1;
	rhsn=2;
	if ((lhs<>lhsn) & (rhs<>rhsn))
		message="[out]=NL_S_AESDecryption(s,in)";
		help('NL_S_AESDecryption');
		error(msprintf("Correct Syntax: %s: Expected %d input and %d output arguments",message,rhsn,lhsn));
	end
	// AES DECRYPTION Decrypt 16-bytes vector.
	// s:                AES structure
	// in:               input 16-bytes vector (ciphertext) -> [0:255]
	// out:              output 16-bytes vector (plaintext)

	// 16 -> 4 x 4
	state = matrix(in, 4, 4);

	// Initial round : AddRoundKey keyexp(s.rounds*4 + (1:4))
	// AddRoundKey(state,RoundKeys[Nr]) ->  
	state = bitxor(state, (s.keyexp(s.rounds*4 + (1:4), :))');

	// Loops (s.rounds - 1) rounds
	for i = (s.rounds - 1):-1:1
		// ShiftRows
		state = NL_S_AESShiftRowsR(state);
		// SubBytes - lookup table
		state = s.inv_s_box(state + 1);
		state=matrix(state,4,4);//change to 4x4
		// AddRoundKey keyexp(i*4 + (1:4))
		state = bitxor(state, (s.keyexp((1:4) + 4*i, :))');
		// MixColumns
		state = NL_S_AESMixColumnsR(state, s);
	end

	// Final round
	// ShiftRows
	state = NL_S_AESShiftRowsR(state);
	// SubBytes - lookup table
	state = s.inv_s_box(state + 1);
	state=matrix(state,4,4);//change to 4x4
	// AddRoundKey keyexp(1:4)
	state = bitxor(state, (s.keyexp(1:4, :))');
	// 4 x 4 -> 16
	out = matrix(state, 1, 16);
endfunction
