function [s]=NL_S_AESInitialization(key)
	[lhs,rhs]=argn();
	lhsn=1;
	rhsn=1;
	if ((lhs<>lhsn)&(rhs<>rhsn))
		message="[s]=NL_S_AESInitialization(key)";
		help('NL_S_AESInitialization');
		error(msprintf("Correct Syntax: %s: Expected %d input and %d output arguments",message,rhsn,lhsn));
	end

	// structure with s-boxes, expanded key, etc.
	// Usage:            s = NL_S_AESInit([23 34 168 ... 39])
	// key:              16 (AES-128), 24 (AES-192), and 32 (AES-256)
	//                   items array with bytes of key
	// s:                AES structure for AES parameters and tables

	key = key(:);
	lk = length(key);
	select (lk)
		case 16 then rounds=10,
		case 24 then rounds=12,
		case 32 then rounds=14,
		else error('Only AES-128, AES-192, and AES-256 are supported.'),
	end
	s = {};
	s.key = key;
	s.bytes = lk;
	s.length = lk * 8;
	s.rounds = rounds;
	// irreducible polynomial for multiplication in a finite field 0x11b
	// bin2dec('100011011');
	s.mod_pol = 283;
	//s-box method
	//logarithm lookup table and its reverse
	aes_logt = zeros(1,256);
	aes_ilogt = zeros(1,256);
	gen = 1;
	for i = 0:255
		aes_logt(gen + 1) = i;
		aes_ilogt(i + 1) = gen;
		gen = NL_S_AESPolynomialMult(gen, 3, s.mod_pol);
	end
	// store log tables
	s.aes_logt = aes_logt;
	s.aes_ilogt = aes_ilogt;	
	// build s-box and its inverse
	s_box = zeros(1,256);
	loctable = [1 2 4 8 16 32 64 128 1 2 4 8 16 32 64 128];
	for i = 0:255
		if (i == 0)
			inve = 0;
		else
			inve = aes_ilogt(255 - aes_logt(i + 1) + 1);
		end
		temp = 0;
		for bi = 0:7
			temp2 = sign(bitand(inve, loctable(bi + 1)));
			temp2 = temp2 + sign(bitand(inve, loctable(bi + 4 + 1)));
			temp2 = temp2 + sign(bitand(inve, loctable(bi + 5 + 1)));
			temp2 = temp2 + sign(bitand(inve, loctable(bi + 6 + 1)));
			temp2 = temp2 + sign(bitand(inve, loctable(bi + 7 + 1)));
			temp2 = temp2 + sign(bitand(99, loctable(bi + 1)));
			//Matlab:rem(X,Y)
			//Scilab:X-fix(X./Y).*Y
			if (temp2-fix(temp2./2).*2)
				temp = bitor(temp, loctable(bi + 1));
			end
		end
		s_box(i + 1) = temp;
	end
	inv_s_box(s_box(1:256) + 1) = (0:255);
	// table correction (must be)
	s_box(1 + 1) = 124;
	inv_s_box(124 + 1) = 1;
	inv_s_box(99 + 1) = 0;
	s.s_box = s_box;
	s.inv_s_box = inv_s_box;
	// tables for fast MixColumns
	mix_col2 = zeros(1,256);
	mix_col3 = mix_col2;
	mix_col9 = mix_col2;
	mix_col11 = mix_col2;
	mix_col13 = mix_col2;
	mix_col14 = mix_col2;
	for i = 1:256
		mix_col2(i) = NL_S_AESPolynomialMult(2, i - 1, s.mod_pol);
		mix_col3(i) = NL_S_AESPolynomialMult(3, i - 1, s.mod_pol);
		mix_col9(i) = NL_S_AESPolynomialMult(9, i - 1, s.mod_pol);
		mix_col11(i) = NL_S_AESPolynomialMult(11, i - 1, s.mod_pol);
		mix_col13(i) = NL_S_AESPolynomialMult(13, i - 1, s.mod_pol);
		mix_col14(i) = NL_S_AESPolynomialMult(14, i - 1, s.mod_pol);
	end
	s.mix_col2 = mix_col2;
	s.mix_col3 = mix_col3;
	s.mix_col9 = mix_col9;
	s.mix_col11 = mix_col11;
	s.mix_col13 = mix_col13;
	s.mix_col14 = mix_col14;
	// expanded key
	s.keyexp = NL_S_AESKeyExpansion(s.key, s.s_box, s.rounds, s.mod_pol, s.aes_logt, s.aes_ilogt);
	// poly & invpoly
	s.poly_mat = [2 3 1 1;1 2 3 1;1 1 2 3;3 1 1 2];
	s.inv_poly_mat =[14 11 13 9;9 14 11 13;13 9 14 11;11 13 9 14];
endfunction
